/*****************************************************************************
 * vpar_motion.c : motion vectors parsing
 * (c)1999 VideoLAN
 *****************************************************************************/

/*****************************************************************************
 * Preamble
 *****************************************************************************/
#include <errno.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/uio.h>
#include <X11/Xlib.h>
#include <X11/extensions/XShm.h>

#include "config.h"
#include "common.h"
#include "mtime.h"
#include "vlc_thread.h"

#include "intf_msg.h"
#include "debug.h"                    /* ?? temporaire, requis par netlist.h */

#include "input.h"
#include "input_netlist.h"
#include "decoder_fifo.h"
#include "video.h"
#include "video_output.h"

#include "vdec_idct.h"
#include "video_decoder.h"
#include "vdec_motion.h"

#include "vpar_blocks.h"
#include "vpar_headers.h"
#include "video_fifo.h"
#include "vpar_synchro.h"
#include "video_parser.h"

#define MAX_COUNT 3

/*
 * Local prototypes
 */

/*****************************************************************************
 * vpar_SynchroUpdateTab : Update a mean table in the synchro structure
 *****************************************************************************/
double vpar_SynchroUpdateTab( video_synchro_tab_t * tab, int count )
{
    if( tab->count < MAX_COUNT)
        tab->count++;

    tab->mean = ( (tab->count-1) * tab->mean + count )
                    / tab->count;

    tab->deviation = ( (tab->count-1) * tab->deviation
                    + abs (tab->mean - count) ) / tab->count;

}

/*****************************************************************************
 * vpar_SynchroUpdateStructures : Update the synchro structures
 *****************************************************************************/
void vpar_SynchroUpdateStructures( video_synchro_tab_t * tab,
                                   int i_coding_type )
{
    double candidate_deviation;
    double optimal_deviation;
    double predict;

    switch(i_coding_type)
    {
        case P_CODING_TYPE:
            p_vpar->synchro.current_p_count++;
            break;
        case B_CODING_TYPE:
            p_vpar->synchro.current_b_count++;
            break;
        case I_CODING_TYPE:

            /* update all the structures for P images */
            optimal_deviation = vpar_SynchroUpdateTab(
                            &p_vpar->synchro.tab_p[0],
                            p_vpar->synchro.current_p_count);
            predict = p_vpar->synchro.tab_p[0].mean;

            candidate_deviation = vpar_SynchroUpdateTab(
                            &p_vpar->synchro.tab_p[1 + (modulo & 0x1)],
                            p_vpar->synchro.current_p_count);
            if (candidate_deviation < optimal_deviation)
	    {
                optimal_deviation = candidate_deviation;
                predict = p_vpar->synchro.tab_p[1 + (modulo & 0x1)].mean;
            }

            candidate_deviation = vpar_SynchroUpdateTab(
                            &p_vpar->synchro.tab_p[3 + (modulo % 3)],
                            p_vpar->synchro.current_p_count);
            if (candidate_deviation < optimal_deviation)
	    {
                optimal_deviation = candidate_deviation;
                predict = p_vpar->synchro.tab_p[1 + (modulo & 0x1)].mean;
            }

	    p_vpar->synchro.p_count_predict = predict;


            /* update all the structures for B images */
            optimal_deviation = vpar_SynchroUpdateTab(
                            &p_vpar->synchro.tab_b[0],
                            p_vpar->synchro.current_b_count);
            predict = p_vpar->synchro.tab_b[0].mean;

            candidate_deviation = vpar_SynchroUpdateTab(
                            &p_vpar->synchro.tab_b[1 + (modulo & 0x1)],
                            p_vpar->synchro.current_b_count);
            if (candidate_deviation < optimal_deviation)
	    {
                optimal_deviation = candidate_deviation;
                predict = p_vpar->synchro.tab_b[1 + (modulo & 0x1)].mean;
            }

            candidate_deviation = vpar_SynchroUpdateTab(
                            &p_vpar->synchro.tab_b[3 + (modulo % 3)],
                            p_vpar->synchro.current_b_count);
            if (candidate_deviation < optimal_deviation)
	    {
                optimal_deviation = candidate_deviation;
                predict = p_vpar->synchro.tab_b[1 + (modulo & 0x1)].mean;
            }

	    p_vpar->synchro.b_count_predict = predict;


            break;
    }

    p_vpar->synchro.modulo++;
}

/*****************************************************************************
 * vpar_SynchroChoose : Decide whether we will decode a picture or not
 *****************************************************************************/
boolean_t vpar_SynchroChoose( vpar_thread_t * p_vpar, int i_coding_type,
                              int i_structure )
{
//    return( 1 );
    return( i_coding_type == I_CODING_TYPE || i_coding_type == P_CODING_TYPE );
    //return( i_coding_type == I_CODING_TYPE );
}

/*****************************************************************************
 * vpar_SynchroTrash : Update timers when we trash a picture
 *****************************************************************************/
void vpar_SynchroTrash( vpar_thread_t * p_vpar, int i_coding_type,
                        int i_structure )
{
    vpar_SynchroUpdateStructures (p_vpar, i_coding_type, i_structure);

}

/*****************************************************************************
 * vpar_SynchroDecode : Update timers when we decide to decode a picture
 *****************************************************************************/
mtime_t vpar_SynchroDecode( vpar_thread_t * p_vpar, int i_coding_type,
                            int i_structure )
{
    vpar_SynchroUpdateStructures (p_vpar, i_coding_type, i_structure);

    return mdate() + 3000000;
}

/*****************************************************************************
 * vpar_SynchroEnd : Called when the image is totally decoded
 *****************************************************************************/
void vpar_SynchroEnd( vpar_thread_t * p_vpar )
{

}

